local ADDON_NAME, namespace = ... 	--localization
local L = namespace.L 				--localization
local version = GetAddOnMetadata(ADDON_NAME, "Version")
local addoninfo = 'v'..version
--------------------------
-- SavedVariables Setup --
--------------------------
local DejaClassicStats, gdbprivate = ...

gdbprivate.gdbdefaults = {
}
gdbprivate.gdbdefaults.gdbdefaults = {
}

----------------------------
-- Saved Variables Loader --
----------------------------
local loader = CreateFrame("Frame")
	loader:RegisterEvent("ADDON_LOADED")
	loader:SetScript("OnEvent", function(self, event, arg1)
		if event == "ADDON_LOADED" and arg1 == "DejaClassicStats" then
			local function initDB(gdb, gdbdefaults)
				if type(gdb) ~= "table" then gdb = {} end
				if type(gdbdefaults) ~= "table" then return gdb end
				for k, v in pairs(gdbdefaults) do
					if type(v) == "table" then
						gdb[k] = initDB(gdb[k], v)
					elseif type(v) ~= type(gdb[k]) then
						gdb[k] = v
					end
				end
				return gdb
			end

			DejaClassicStatsDB = initDB(DejaClassicStatsDB, gdbprivate.gdbdefaults) --the first per account saved variable. The second per-account variable DCS_ClassSpecDB is handled in DCS_Layouts.lua
			gdbprivate.gdb = DejaClassicStatsDB --fast access for checkbox states
			self:UnregisterEvent("ADDON_LOADED")
		end
	end)

local DejaClassicStats, private = ...

private.defaults = {
}
private.defaults.dcsdefaults = {
}

DejaClassicStats = {};

----------------------------
-- Saved Variables Loader --
----------------------------
local loader = CreateFrame("Frame")
	loader:RegisterEvent("ADDON_LOADED")
	loader:SetScript("OnEvent", function(self, event, arg1)
		if event == "ADDON_LOADED" and arg1 == "DejaClassicStats" then
			local function initDB(db, defaults)
				if type(db) ~= "table" then db = {} end
				if type(defaults) ~= "table" then return db end
				for k, v in pairs(defaults) do
					if type(v) == "table" then
						db[k] = initDB(db[k], v)
					elseif type(v) ~= type(db[k]) then
						db[k] = v
					end
				end
				return db
			end

			DejaClassicStatsDBPC = initDB(DejaClassicStatsDBPC, private.defaults) --saved variable per character, currently not used.
			private.db = DejaClassicStatsDBPC

			self:UnregisterEvent("ADDON_LOADED")
		end
	end)

-- Uncomment below the following three database saved variables setup lines for DejaView integration.
-- SavedVariables Setup
-- local DejaClassicStats, private = ...
-- private.defaults = {}
-- DejaClassicStats = {};

---------------------
-- DCS Slash Setup --
---------------------
local RegisteredEvents = {};
local dcsslash = CreateFrame("Frame", "DejaClassicStatsSlash", UIParent)

dcsslash:SetScript("OnEvent", function (self, event, ...) 
	if (RegisteredEvents[event]) then 
	return RegisteredEvents[event](self, event, ...) 
	end
end)

function RegisteredEvents:ADDON_LOADED(event, addon, ...)
	if (addon == "DejaClassicStats") then
		--SLASH_DejaClassicStats1 = (L["/dcstats"])
		SLASH_DejaClassicStats1 = "/dcstats"
		SlashCmdList["DejaClassicStats"] = function (msg, editbox)
			DejaClassicStats.SlashCmdHandler(msg, editbox)	
	end
	--	DEFAULT_CHAT_FRAME:AddMessage("DejaClassicStats loaded successfully. For options: Esc>Interface>AddOns or type /dcstats.",0,192,255)
	end
end

for k, v in pairs(RegisteredEvents) do
	dcsslash:RegisterEvent(k)
end

function DejaClassicStats.ShowHelp()
	print(addoninfo)
	print(L["DejaClassicStats Slash commands (/dcstats):"])
	print(L["  /dcstats config: Opens the DejaClassicStats addon config menu."])
	print(L["  /dcstats reset:  Resets DejaClassicStats options to default."])
end

--[[
function DejaClassicStats.SetConfigToDefaults()
	print(L["Resetting config to defaults"])
	DejaClassicStatsDBPC = private.db --DBPC not used, when (and if) will, DefaultConfig should be replaced with private.db
	RELOADUI()
end
--]]

--[[
function DejaClassicStats.GetConfigValue(key)
	return DejaClassicStatsDBPC[key] --I think here a loop is required -- Called in the dumpconfig loop in the DejaClassicStats.SlashCmdHandler function.
end
--]]

--[[
function DejaClassicStats.PrintPerformanceData()
	UpdateAddOnMemoryUsage()
	local mem = GetAddOnMemoryUsage("DejaClassicStats")
	print(L["DejaClassicStats is currently using "] .. mem .. L[" kbytes of memory"])
	collectgarbage("collect")
	UpdateAddOnMemoryUsage()
	mem = GetAddOnMemoryUsage("DejaClassicStats")
	print(L["DejaClassicStats is currently using "] .. mem .. L[" kbytes of memory after garbage collection"])
end
--]]

function DejaClassicStats.SlashCmdHandler(msg, editbox)
    msg = string.lower(msg)
	--print("command is " .. msg .. "\n")
	--if (string.lower(msg) == L["config"]) then --I think string.lowermight not work for Russian letters
	if (msg == "config") then
		InterfaceOptionsFrame_OpenToCategory("DejaClassicStats");
		InterfaceOptionsFrame_OpenToCategory("DejaClassicStats");
		InterfaceOptionsFrame_OpenToCategory("DejaClassicStats");
	--[[	
	elseif (string.lower(msg) == L["dumpconfig"]) then
		print(L["With defaults"])
		for k,v in pairs(private.db) do --produces error
			print(k,DejaClassicStats.GetConfigValue(k))
		end
		print(L["Direct table"])
		for k,v in pairs(private.db) do
			print(k,v)
		end
	--]]
	--elseif (string.lower(msg) == L["reset"]) then
	elseif (msg == "reset") then
		--DejaClassicStatsDBPC = private.defaults;
		gdbprivate.gdb.gdbdefaults = gdbprivate.gdbdefaults.gdbdefaults
		ReloadUI();
	--[[	
	elseif (string.lower(msg) == L["perf"]) then
		DejaClassicStats.PrintPerformanceData()
	--]]
	else
		DejaClassicStats.ShowHelp()
	end
end
	SlashCmdList["DejaClassicStats"] = DejaClassicStats.SlashCmdHandler;

-----------------------
-- DCS Options Panel --
-----------------------
DejaClassicStats.panel = CreateFrame( "Frame", "DejaClassicStatsPanel", UIParent );
DejaClassicStats.panel.name = "DejaClassicStats";
InterfaceOptions_AddCategory(DejaClassicStats.panel);

-- DCS, DejaView Child Panel
-- DejaViewPanel.DejaClassicStatsPanel = CreateFrame( "Frame", "DejaClassicStatsPanel", DejaViewPanel);
-- DejaViewPanel.DejaClassicStatsPanel.name = "DejaClassicStats";
-- Specify childness of this panel (this puts it under the little red [+], instead of giving it a normal AddOn category)
-- DejaViewPanel.DejaClassicStatsPanel.parent = DejaViewPanel.name;
-- Add the child to the Interface Options
-- InterfaceOptions_AddCategory(DejaViewPanel.DejaClassicStatsPanel);

local dcstitle=CreateFrame("Frame", "DCSTitle", DejaClassicStatsPanel)
	dcstitle:SetPoint("TOPLEFT", 10, -10)
	--dcstitle:SetScale(2.0)
	dcstitle:SetWidth(300)
	dcstitle:SetHeight(100)
	dcstitle:Show()

local dcstitleFS = dcstitle:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	dcstitleFS:SetText('|cff00c0ffDejaClassicStats|r')
	dcstitleFS:SetPoint("TOPLEFT", 0, 0)
	dcstitleFS:SetFont("Fonts\\FRIZQT__.TTF", 20)

local dcsversionFS = DejaClassicStatsPanel:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	dcsversionFS:SetText('|cff00c0ff' .. addoninfo .. '|r')
	dcsversionFS:SetPoint("BOTTOMRIGHT", -10, 10)
	dcsversionFS:SetFont("Fonts\\FRIZQT__.TTF", 12)
	
local dcsresetcheck = CreateFrame("Button", "DCSResetButton", DejaClassicStatsPanel, "UIPanelButtonTemplate")
	dcsresetcheck:ClearAllPoints()
	dcsresetcheck:SetPoint("BOTTOMLEFT", 5, 5)
	dcsresetcheck:SetScale(1.25)

	--local LOCALE = GetLocale()
	local LOCALE = namespace.locale
		--print (LOCALE)
	local sometable = {
		["ptBR"] = {},
		["frFR"] = {},
		["deDE"] = {},
	}
	if sometable[LOCALE] then
		LOCALE = 175
	else
		--print ("enUS = 125")
		LOCALE = 125
	end
	--[[
	if (LOCALE == "ptBR" or LOCALE == "frFR" or LOCALE == "deDE") then
		--print ("ptBR, frFR, deDE = 175")
		LOCALE = 175
	else
		--print ("enUS = 125")
		LOCALE = 125
	end
	--]]
	dcsresetcheck:SetWidth(LOCALE)

	dcsresetcheck:SetHeight(30)
	_G[dcsresetcheck:GetName() .. "Text"]:SetText(L["Reset to Default"])
	dcsresetcheck:SetScript("OnClick", function(self, button, down)
 		gdbprivate.gdb.gdbdefaults = gdbprivate.gdbdefaults.gdbdefaults;
		ReloadUI();
	end)
		
	----------------------
	-- Panel Categories --
	----------------------
	
	-- --Average Item Level
	-- local dcsILvlPanelCategoryFS = DejaClassicStatsPanel:CreateFontString("dcsILvlPanelCategoryFS", "OVERLAY", "GameFontNormal")
	-- dcsILvlPanelCategoryFS:SetText('|cffffffff' .. L["Average Item Level:"] .. '|r') --wouldn't be more efficient through format?
	-- dcsILvlPanelCategoryFS:SetPoint("TOPLEFT", 25, -40)
	-- dcsILvlPanelCategoryFS:SetFontObject("GameFontNormalLarge") --Use instead of SetFont("Fonts\\FRIZQT__.TTF", 15) or Russian, Korean and Chinese characters won't work.
	
	-- --Character Stats 
	-- local dcsStatsPanelcategoryFS = DejaClassicStatsPanel:CreateFontString("dcsStatsPanelcategoryFS", "OVERLAY", "GameFontNormal")
	-- dcsStatsPanelcategoryFS:SetText('|cffffffff' .. L["Character Stats:"] .. '|r')
	-- dcsStatsPanelcategoryFS:SetPoint("TOPLEFT", 25, -150)
	-- dcsStatsPanelcategoryFS:SetFontObject("GameFontNormalLarge") --Use instead of SetFont("Fonts\\FRIZQT__.TTF", 15) or Russian, Korean and Chinese characters won't work.
	
	--Item Slots
	local dcsItemsPanelCategoryFS = DejaClassicStatsPanel:CreateFontString("dcsItemsPanelCategoryFS", "OVERLAY", "GameFontNormal")
	dcsItemsPanelCategoryFS:SetText('|cffffffff' .. L["Item Slots:"] .. '|r')
	dcsItemsPanelCategoryFS:SetPoint("TOPLEFT", 25, -40)
	dcsItemsPanelCategoryFS:SetFontObject("GameFontNormalLarge") --Use instead of SetFont("Fonts\\FRIZQT__.TTF", 15) or Russian, Korean and Chinese characters won't work.
	
	-- --Miscellaneous
	-- local dcsMiscPanelCategoryFS = DejaClassicStatsPanel:CreateFontString("dcsMiscPanelCategoryFS", "OVERLAY", "GameFontNormal")
	-- dcsMiscPanelCategoryFS:SetText('|cffffffff' .. L["Miscellaneous:"] .. '|r')
	-- dcsMiscPanelCategoryFS:SetPoint("LEFT", 25, -165)
	-- dcsMiscPanelCategoryFS:SetFontObject("GameFontNormalLarge") --Use instead of SetFont("Fonts\\FRIZQT__.TTF", 15) or Russian, Korean and Chinese characters won't work.
	
CharacterAttributesFrame:Hide()

local DCS_FrameWidth = 200
local DCS_RframeInset = 25

local StatScrollFrame = CreateFrame("ScrollFrame", nil, CharacterFrame, "UIPanelScrollFrameTemplate")
	StatScrollFrame:ClearAllPoints()
	StatScrollFrame:SetSize( DCS_FrameWidth, 424 )
	StatScrollFrame:SetPoint("TOPLEFT", "CharacterFrame", "TOPRIGHT", -40, -14)
	StatScrollFrame:SetFrameStrata("BACKGROUND")
	StatScrollFrame.ScrollBar:ClearAllPoints()
	StatScrollFrame.ScrollBar:SetPoint("TOPLEFT", StatScrollFrame, "TOPRIGHT", -18, -16)
	StatScrollFrame.ScrollBar:SetPoint("BOTTOMLEFT", StatScrollFrame, "BOTTOMRIGHT", -18, 16)
	StatScrollFrame.ScrollBar:Show()
	
	StatScrollFrame:HookScript("OnScrollRangeChanged", function(self, xrange, yrange)
		self.ScrollBar:SetShown(floor(yrange) ~= 0)
	end)

local DejaClassicStatsPane = CreateFrame("Frame", "DejaClassicStatsPane", CharacterFrame)
	DejaClassicStatsPane:SetSize( DCS_FrameWidth, 500 )
	DejaClassicStatsPane:SetPoint("TOPLEFT", "CharacterFrame", "TOPRIGHT", -40, -14)
	DejaClassicStatsPane:SetFrameStrata("BACKGROUND")
	-- DejaClassicStatsPane:Show()

local t=DejaClassicStatsPane:CreateTexture(nil,"ARTWORK")
	t:SetAllPoints(DejaClassicStatsPane)
	t:SetColorTexture(0, 0, 0, 1)
	-- t:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopRight");
	-- t:SetTexCoord(0, 1, 0, 1);

	StatScrollFrame:SetScrollChild(DejaClassicStatsPane)

local className, classFilename, classID = UnitClass("player") --Players Class Color
local rPerc, gPerc, bPerc, argbHex = GetClassColor(classFilename)
-- print(classFilename,rPerc, gPerc, bPerc, argbHex)

-----------
--Primary--
-----------
local DCSPrimaryStatsHeader = CreateFrame("Frame", "DCSPrimaryStatsHeader", DejaClassicStatsPane)
	DCSPrimaryStatsHeader:SetSize( DCS_FrameWidth, 20 )
	DCSPrimaryStatsHeader:SetPoint("TOP", "DejaClassicStatsPane", "TOP", 0, -10)
	-- DCSPrimaryStatsHeader:SetFrameStrata("BACKGROUND")
	-- DCSPrimaryStatsHeader:Show()

local DCSPrimaryStatsFS = DCSPrimaryStatsHeader:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	DCSPrimaryStatsFS:SetText('Primary')
	DCSPrimaryStatsFS:SetTextColor(rPerc, gPerc, bPerc)
	DCSPrimaryStatsFS:SetPoint("CENTER", 0, 0)
	DCSPrimaryStatsFS:SetFont("Fonts\\FRIZQT__.TTF", 18)
	DCSPrimaryStatsFS:SetJustifyH("CENTER")

local t=DCSPrimaryStatsHeader:CreateTexture(nil,"ARTWORK")
	t:SetAllPoints(DCSPrimaryStatsHeader)
	t:SetColorTexture(1, 1, 1, 0)
	-- t:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopRight");
	-- t:SetTexCoord(0, 1, 0, 1);

-- Update our primary stats (Strength, Agility, etc.).
function PaperDollFrame_SetPrimaryStats()
	for i=1, NUM_STATS, 1 do
		local text = _G["CharacterStatFrame"..i.."StatText"];
		local frame = _G["CharacterStatFrame"..i];
		local stat;
		local effectiveStat;
		local posBuff;
		local negBuff;
		stat, effectiveStat, posBuff, negBuff = UnitStat("player", i);

		-- Set the tooltip text
		local tooltipText = HIGHLIGHT_FONT_COLOR_CODE.._G["SPELL_STAT"..i.."_NAME"].." ";
		
		-- Get class specific tooltip for that stat
		local temp, classFileName = UnitClass("player");
		local classStatText = _G[strupper(classFileName).."_"..frame.stat.."_".."TOOLTIP"];
		-- If can't find one use the default
		if ( not classStatText ) then
			classStatText = _G["DEFAULT".."_"..frame.stat.."_".."TOOLTIP"];
		end

		if ( ( posBuff == 0 ) and ( negBuff == 0 ) ) then
			text:SetText(effectiveStat);
			frame.tooltip = tooltipText..effectiveStat..FONT_COLOR_CODE_CLOSE;
			frame.tooltip2 = classStatText;
		else 
			tooltipText = tooltipText..effectiveStat;
			if ( posBuff > 0 or negBuff < 0 ) then
				tooltipText = tooltipText.." ("..(stat - posBuff - negBuff)..FONT_COLOR_CODE_CLOSE;
			end
			if ( posBuff > 0 ) then
				tooltipText = tooltipText..FONT_COLOR_CODE_CLOSE..GREEN_FONT_COLOR_CODE.."+"..posBuff..FONT_COLOR_CODE_CLOSE;
			end
			if ( negBuff < 0 ) then
				tooltipText = tooltipText..RED_FONT_COLOR_CODE.." "..negBuff..FONT_COLOR_CODE_CLOSE;
			end
			if ( posBuff > 0 or negBuff < 0 ) then
				tooltipText = tooltipText..HIGHLIGHT_FONT_COLOR_CODE..")"..FONT_COLOR_CODE_CLOSE;
			end
			frame.tooltip = tooltipText;
			frame.tooltip2= classStatText;
			
			-- If there are any negative buffs then show the main number in red even if there are
			-- positive buffs. Otherwise show in green.
			if ( negBuff < 0 ) then
				text:SetText(RED_FONT_COLOR_CODE..effectiveStat..FONT_COLOR_CODE_CLOSE);
			else
				text:SetText(GREEN_FONT_COLOR_CODE..effectiveStat..FONT_COLOR_CODE_CLOSE);
			end
		end
		local scale = 1.5
		frame:SetParent(DejaClassicStatsPane)
		frame:SetScale(1.5)
		frame:SetWidth( (DCS_FrameWidth / scale) - DCS_RframeInset)
		if (i==1) then
			frame:SetPoint("TOPLEFT", DCSPrimaryStatsHeader, "BOTTOMLEFT", 10,0)--Not TOP and BOTTOM becasue of a parenting issue that centers it over the Characterframe...
		else
			frame:SetPoint("TOP", _G["CharacterStatFrame"..(i-1)], "BOTTOM", 0,0)
		end
		--print(tooltipText)
	end
end


function PaperDollFrame_SetArmor(unit, prefix)
	if ( not unit ) then
		unit = "player";
	end
	if ( not prefix ) then
		prefix = "Character";
	end

	local base, effectiveArmor, armor, posBuff, negBuff = UnitArmor(unit);

	if (unit ~= "player") then
		--[[ In 1.12.0, UnitArmor didn't report positive / negative buffs for units that weren't the active player.
			 This hack replicates that behavior for the UI. ]]
		base = effectiveArmor;
		armor = effectiveArmor;
		posBuff = 0;
		negBuff = 0;
	end

	local totalBufs = posBuff + negBuff;

	local frame = _G[prefix.."ArmorFrame"];
	local text = _G[prefix.."ArmorFrameStatText"];

	PaperDollFormatStat(ARMOR, base, posBuff, negBuff, frame, text);
	local playerLevel = UnitLevel(unit);
	local armorReduction = effectiveArmor/((85 * playerLevel) + 400);
	armorReduction = 100 * (armorReduction/(armorReduction + 1));
	
	frame.tooltip2 = format(ARMOR_TOOLTIP, playerLevel, armorReduction);

	local scale = 1.5
	frame:SetParent(DejaClassicStatsPane)
	frame:SetScale(1.5)
	frame:SetWidth( (DCS_FrameWidth / scale) - DCS_RframeInset)
	frame:SetPoint("TOP", CharacterStatFrame5, "BOTTOM", 0,0)
	--print("Armor", effectiveArmor)
end

-----------
--Offense--
-----------
local DCSOffensiveStatsHeader = CreateFrame("Frame", "DCSOffensiveStatsHeader", DejaClassicStatsPane)
	DCSOffensiveStatsHeader:SetSize( DCS_FrameWidth, 20 )
	DCSOffensiveStatsHeader:SetPoint("TOP", "DejaClassicStatsPane", "TOP", 0, -150)
	-- DCSOffensiveStatsHeader:SetFrameStrata("BACKGROUND")
	-- DCSOffensiveStatsHeader:Show()

local DCSOffensiveStatsFS = DCSOffensiveStatsHeader:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	DCSOffensiveStatsFS:SetText('Offense')
	DCSOffensiveStatsFS:SetTextColor(rPerc, gPerc, bPerc)
	DCSOffensiveStatsFS:SetPoint("CENTER", 0, 0) --This is -2 to center the header "Offense" better.
	DCSOffensiveStatsFS:SetFont("Fonts\\FRIZQT__.TTF", 18)
	DCSOffensiveStatsFS:SetJustifyH("CENTER")

local t=DCSOffensiveStatsHeader:CreateTexture(nil,"ARTWORK")
	t:SetAllPoints(DCSOffensiveStatsHeader)
	t:SetColorTexture(1, 1, 1, 0)
	-- t:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopRight");
	-- t:SetTexCoord(0, 1, 0, 1);

-- Note: while this function was historically named "BothHands",
-- it looks like it only ever displayed attack rating for the main hand.
function PaperDollFrame_SetAttackBothHands(unit, prefix)
	if ( not unit ) then
		unit = "player";
	end
	if ( not prefix ) then
		prefix = "Character";
	end

	local mainHandAttackBase, mainHandAttackMod = UnitAttackBothHands(unit);

	local frame = _G[prefix.."AttackFrame"];
	local text = _G[prefix.."AttackFrameStatText"];

	if( mainHandAttackMod == 0 ) then
		text:SetText(mainHandAttackBase);
	else
		local color = RED_FONT_COLOR_CODE;
		if( mainHandAttackMod > 0 ) then
			color = GREEN_FONT_COLOR_CODE;
		end
		text:SetText(color..(mainHandAttackBase + mainHandAttackMod)..FONT_COLOR_CODE_CLOSE);
	end

	frame.tooltip = ATTACK_TOOLTIP;
	frame.tooltip2 = ATTACK_TOOLTIP_SUBTEXT;

	local scale = 1.5
	frame:SetParent(DejaClassicStatsPane)
	frame:SetScale(1.5)
	frame:SetWidth( (DCS_FrameWidth / scale) - DCS_RframeInset)
	frame:SetPoint("TOPLEFT", DCSOffensiveStatsHeader, "BOTTOMLEFT", 10,0)--Not TOP and BOTTOM becasue of a parenting issue that centers it over the Characterframe...
	--print(frame.tooltip)
end

function PaperDollFrame_SetAttackPower(unit, prefix)
	if ( not unit ) then
		unit = "player";
	end
	if ( not prefix ) then
		prefix = "Character";
	end
	
	local base, posBuff, negBuff = UnitAttackPower(unit);

	local frame = _G[prefix.."AttackPowerFrame"]; 
	local text = _G[prefix.."AttackPowerFrameStatText"];

	PaperDollFormatStat(MELEE_ATTACK_POWER, base, posBuff, negBuff, frame, text);
	frame.tooltip2 = format(MELEE_ATTACK_POWER_TOOLTIP, max((base+posBuff+negBuff), 0)/ATTACK_POWER_MAGIC_NUMBER);

	local scale = 1.5
	frame:SetParent(DejaClassicStatsPane)
	frame:SetScale(1.5)
	frame:SetWidth( ( (DCS_FrameWidth / scale) - DCS_RframeInset) -10) --Inset 10 becasue it is a substat
	frame:SetPoint("TOPLEFT", CharacterAttackFrame, "BOTTOMLEFT", 10,0)--Inset 10 as it is a sub stat
	--print(frame.tooltip)
end

function PaperDollFrame_SetDamage(unit, prefix)
	if ( not unit ) then
		unit = "player";
	end
	if ( not prefix ) then
		prefix = "Character";
	end

	local damageText = _G[prefix.."DamageFrameStatText"];
	local damageFrame = _G[prefix.."DamageFrame"];

	local speed, offhandSpeed = UnitAttackSpeed(unit);
	
	local minDamage;
	local maxDamage; 
	local minOffHandDamage;
	local maxOffHandDamage; 
	local physicalBonusPos;
	local physicalBonusNeg;
	local percent;
	minDamage, maxDamage, minOffHandDamage, maxOffHandDamage, physicalBonusPos, physicalBonusNeg, percent = UnitDamage(unit);
	local displayMin = max(floor(minDamage),1);
	local displayMax = max(ceil(maxDamage),1);

	minDamage = (minDamage / percent) - physicalBonusPos - physicalBonusNeg;
	maxDamage = (maxDamage / percent) - physicalBonusPos - physicalBonusNeg;

	local baseDamage = (minDamage + maxDamage) * 0.5;
	local fullDamage = (baseDamage + physicalBonusPos + physicalBonusNeg) * percent;
	local totalBonus = (fullDamage - baseDamage);
	local damagePerSecond = (max(fullDamage,1) / speed);
	local damageTooltip = max(floor(minDamage),1).." - "..max(ceil(maxDamage),1);
	
	local colorPos = "|cff20ff20";
	local colorNeg = "|cffff2020";
	if ( totalBonus == 0 ) then
		if ( ( displayMin < 100 ) and ( displayMax < 100 ) ) then 
			damageText:SetText(displayMin.." - "..displayMax);	
		else
			damageText:SetText(displayMin.."-"..displayMax);
		end
	else
		
		local color;
		if ( totalBonus > 0 ) then
			color = colorPos;
		else
			color = colorNeg;
		end
		if ( ( displayMin < 100 ) and ( displayMax < 100 ) ) then 
			damageText:SetText(color..displayMin.." - "..displayMax.."|r");	
		else
			damageText:SetText(color..displayMin.."-"..displayMax.."|r");
		end
		if ( physicalBonusPos > 0 ) then
			damageTooltip = damageTooltip..colorPos.." +"..physicalBonusPos.."|r";
		end
		if ( physicalBonusNeg < 0 ) then
			damageTooltip = damageTooltip..colorNeg.." "..physicalBonusNeg.."|r";
		end
		if ( percent > 1 ) then
			damageTooltip = damageTooltip..colorPos.." x"..floor(percent*100+0.5).."%|r";
		elseif ( percent < 1 ) then
			damageTooltip = damageTooltip..colorNeg.." x"..floor(percent*100+0.5).."%|r";
		end
		
	end
	damageFrame.damage = damageTooltip;
	damageFrame.attackSpeed = speed;
	damageFrame.dps = damagePerSecond;
	
	-- If there's an offhand speed then add the offhand info to the tooltip
	if ( offhandSpeed ) then
		minOffHandDamage = (minOffHandDamage / percent) - physicalBonusPos - physicalBonusNeg;
		maxOffHandDamage = (maxOffHandDamage / percent) - physicalBonusPos - physicalBonusNeg;

		local offhandBaseDamage = (minOffHandDamage + maxOffHandDamage) * 0.5;
		local offhandFullDamage = (offhandBaseDamage + physicalBonusPos + physicalBonusNeg) * percent;
		local offhandDamagePerSecond = (max(offhandFullDamage,1) / offhandSpeed);
		local offhandDamageTooltip = max(floor(minOffHandDamage),1).." - "..max(ceil(maxOffHandDamage),1);
		if ( physicalBonusPos > 0 ) then
			offhandDamageTooltip = offhandDamageTooltip..colorPos.." +"..physicalBonusPos.."|r";
		end
		if ( physicalBonusNeg < 0 ) then
			offhandDamageTooltip = offhandDamageTooltip..colorNeg.." "..physicalBonusNeg.."|r";
		end
		if ( percent > 1 ) then
			offhandDamageTooltip = offhandDamageTooltip..colorPos.." x"..floor(percent*100+0.5).."%|r";
		elseif ( percent < 1 ) then
			offhandDamageTooltip = offhandDamageTooltip..colorNeg.." x"..floor(percent*100+0.5).."%|r";
		end
		damageFrame.offhandDamage = offhandDamageTooltip;
		damageFrame.offhandAttackSpeed = offhandSpeed;
		damageFrame.offhandDps = offhandDamagePerSecond;
	else
		damageFrame.offhandAttackSpeed = nil;
	end
	local scale = 1.5
	damageFrame:SetParent(DejaClassicStatsPane)
	damageFrame:SetScale(1.5)
	damageFrame:SetWidth( ( (DCS_FrameWidth / scale) - DCS_RframeInset) -10) --Inset 10 becasue it is a substat
	damageFrame:SetPoint("TOPLEFT", CharacterAttackPowerFrame, "BOTTOMLEFT", 0,0)
	--print(damageFrame.offhandAttackSpeed)
end

function PaperDollFrame_SetRangedAttack(unit, prefix)
	if ( not unit ) then
		unit = "player";
	elseif ( unit == "pet" ) then
		return;
	end
	if ( not prefix ) then
		prefix = "Character";
	end

	local hasRelic = UnitHasRelicSlot(unit);

	local rangedAttackBase, rangedAttackMod = UnitRangedAttack(unit);
	local frame = _G[prefix.."RangedAttackFrame"]; 
	local text = _G[prefix.."RangedAttackFrameStatText"];

	-- If no ranged texture then set stats to n/a
	local rangedTexture = GetInventoryItemTexture("player", 18);
	local oldValue = PaperDollFrame.noRanged;
	if ( rangedTexture and not hasRelic ) then
		PaperDollFrame.noRanged = nil;
	else
		text:SetText(NOT_APPLICABLE);
		PaperDollFrame.noRanged = 1;
		frame.tooltip = nil;
	end
	if ( not rangedTexture or hasRelic ) then
		return;
	end
	
	if( rangedAttackMod == 0 ) then
		text:SetText(rangedAttackBase);
	else
		local color = RED_FONT_COLOR_CODE;
		if( rangedAttackMod > 0 ) then
			color = GREEN_FONT_COLOR_CODE;
		end
		text:SetText(color..(rangedAttackBase + rangedAttackMod)..FONT_COLOR_CODE_CLOSE);
	end

	frame.tooltip = RANGED_ATTACK_TOOLTIP;
	frame.tooltip2 = ATTACK_TOOLTIP_SUBTEXT;

	local scale = 1.5
	frame:SetParent(DejaClassicStatsPane)
	frame:SetScale(1.5)
	frame:SetWidth( ( (DCS_FrameWidth / scale) - DCS_RframeInset) ) 
	frame:SetPoint("TOPLEFT", DCSOffensiveStatsHeader, "BOTTOMLEFT", 10,-40)
	-- print(frame.tooltip)
end

function PaperDollFrame_SetRangedAttackPower(unit, prefix)
	if ( not unit ) then
		unit = "player";
	elseif ( unit == "pet" ) then
		return;
	end
	if ( not prefix ) then
		prefix = "Character";
	end
	local frame = getglobal(prefix.."RangedAttackPowerFrame"); 
	local text = getglobal(prefix.."RangedAttackPowerFrameStatText");
	
	-- If no ranged attack then set to n/a
	if ( PaperDollFrame.noRanged ) then
		text:SetText(NOT_APPLICABLE);
		frame.tooltip = nil;
		return;
	end
	if ( HasWandEquipped() ) then
		text:SetText("--");
		frame.tooltip = nil;
		return;
	end

	local base, posBuff, negBuff = UnitRangedAttackPower(unit);
	PaperDollFormatStat(RANGED_ATTACK_POWER, base, posBuff, negBuff, frame, text);
	frame.tooltip2 = format(RANGED_ATTACK_POWER_TOOLTIP, base/ATTACK_POWER_MAGIC_NUMBER);

	local scale = 1.5
	frame:SetParent(DejaClassicStatsPane)
	frame:SetScale(1.5)
	frame:SetWidth( ( (DCS_FrameWidth / scale) - DCS_RframeInset) -10) --Inset 10 becasue it is a substat
	frame:SetPoint("TOPLEFT", CharacterRangedAttackFrame, "BOTTOMLEFT", 10, 0)--Inset 10 as it is a sub stat
	-- print(frame.tooltip)
end

function PaperDollFrame_SetRangedDamage(unit, prefix)
	if ( not unit ) then
		unit = "player";
	elseif ( unit == "pet" ) then
		return;
	end
	if ( not prefix ) then
		prefix = "Character";
	end

	local damageText = getglobal(prefix.."RangedDamageFrameStatText");
	local damageFrame = getglobal(prefix.."RangedDamageFrame");

	-- If no ranged attack then set to n/a
	if ( PaperDollFrame.noRanged ) then
		damageText:SetText(NOT_APPLICABLE);
		damageFrame.damage = nil;
		return;
	end

	local rangedAttackSpeed, minDamage, maxDamage, physicalBonusPos, physicalBonusNeg, percent = UnitRangedDamage(unit);
	local displayMin = max(floor(minDamage),1);
	local displayMax = max(ceil(maxDamage),1);

	minDamage = (minDamage / percent) - physicalBonusPos - physicalBonusNeg;
	maxDamage = (maxDamage / percent) - physicalBonusPos - physicalBonusNeg;

	local baseDamage = (minDamage + maxDamage) * 0.5;
	local fullDamage = (baseDamage + physicalBonusPos + physicalBonusNeg) * percent;
	local totalBonus = (fullDamage - baseDamage);
	local damagePerSecond = (max(fullDamage,1) / rangedAttackSpeed);
	local tooltip = max(floor(minDamage),1).." - "..max(ceil(maxDamage),1);

	if ( totalBonus == 0 ) then
		if ( ( displayMin < 100 ) and ( displayMax < 100 ) ) then 
			damageText:SetText(displayMin.." - "..displayMax);	
		else
			damageText:SetText(displayMin.."-"..displayMax);
		end
	else
		local colorPos = "|cff20ff20";
		local colorNeg = "|cffff2020";
		local color;
		if ( totalBonus > 0 ) then
			color = colorPos;
		else
			color = colorNeg;
		end
		if ( ( displayMin < 100 ) and ( displayMax < 100 ) ) then 
			damageText:SetText(color..displayMin.." - "..displayMax.."|r");	
		else
			damageText:SetText(color..displayMin.."-"..displayMax.."|r");
		end
		if ( physicalBonusPos > 0 ) then
			tooltip = tooltip..colorPos.." +"..physicalBonusPos.."|r";
		end
		if ( physicalBonusNeg < 0 ) then
			tooltip = tooltip..colorNeg.." "..physicalBonusNeg.."|r";
		end
		if ( percent > 1 ) then
			tooltip = tooltip..colorPos.." x"..floor(percent*100+0.5).."%|r";
		elseif ( percent < 1 ) then
			tooltip = tooltip..colorNeg.." x"..floor(percent*100+0.5).."%|r";
		end
		damageFrame.tooltip = tooltip.." "..format(DPS_TEMPLATE, damagePerSecond);
	end
	damageFrame.attackSpeed = rangedAttackSpeed;
	damageFrame.damage = tooltip;
	damageFrame.dps = damagePerSecond;

	local scale = 1.5
	damageFrame:SetParent(DejaClassicStatsPane)
	damageFrame:SetScale(1.5)
	damageFrame:SetWidth( ( (DCS_FrameWidth / scale) - DCS_RframeInset) -10) --Inset 10 becasue it is a substat
	damageFrame:SetPoint("TOPLEFT", CharacterRangedAttackPowerFrame, "BOTTOMLEFT", 0, 0)
	-- print(damageFrame.damage)
end

-----------
--Defense--
-----------
local DCSDefensiveStatsHeader = CreateFrame("Frame", "DCSDefensiveStatsHeader", DejaClassicStatsPane)
	DCSDefensiveStatsHeader:SetSize( DCS_FrameWidth, 20 )
	DCSDefensiveStatsHeader:SetPoint("TOP", "DejaClassicStatsPane", "TOP", 0, -290)
	-- DCSDefensiveStatsHeader:SetFrameStrata("BACKGROUND")
	-- DCSDefensiveStatsHeader:Show()

local DCSDefensiveStatsFS = DCSDefensiveStatsHeader:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	DCSDefensiveStatsFS:SetText('Defense')
	DCSDefensiveStatsFS:SetTextColor(rPerc, gPerc, bPerc)
	DCSDefensiveStatsFS:SetPoint("CENTER", 0, 0) --This is -2 to center the header "Offense" better.
	DCSDefensiveStatsFS:SetFont("Fonts\\FRIZQT__.TTF", 18)
	DCSDefensiveStatsFS:SetJustifyH("CENTER")

local t=DCSDefensiveStatsHeader:CreateTexture(nil,"ARTWORK")
	t:SetAllPoints(DCSDefensiveStatsHeader)
	t:SetColorTexture(1, 1, 1, 0)
	-- t:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-General-TopRight");
	-- t:SetTexCoord(0, 1, 0, 1);

function PaperDollFrame_SetResistances()
	for i=1, NUM_RESISTANCE_TYPES, 1 do
		local resistance;
		local positive;
		local negative;
		local base;
		local text = _G["MagicResText"..i];
		local frame = _G["MagicResFrame"..i];
		
		base, resistance, positive, negative = UnitResistance("player", frame:GetID());

		-- resistances can now be negative. Show Red if negative, Green if positive, white otherwise
		if( abs(negative) > positive ) then
			text:SetText(RED_FONT_COLOR_CODE..resistance..FONT_COLOR_CODE_CLOSE);
		elseif( abs(negative) == positive ) then
			text:SetText(resistance);
		else
			text:SetText(GREEN_FONT_COLOR_CODE..resistance..FONT_COLOR_CODE_CLOSE);
		end

		local resistanceName = _G["RESISTANCE"..(frame:GetID()).."_NAME"];
		frame.tooltip = resistanceName.." "..resistance;
		if ( positive ~= 0 or negative ~= 0 ) then
			-- Otherwise build up the formula
			frame.tooltip = frame.tooltip.. " ( "..HIGHLIGHT_FONT_COLOR_CODE..base;
			if( positive > 0 ) then
				frame.tooltip = frame.tooltip..GREEN_FONT_COLOR_CODE.." +"..positive;
			end
			if( negative < 0 ) then
				frame.tooltip = frame.tooltip.." "..RED_FONT_COLOR_CODE..negative;
			end
			frame.tooltip = frame.tooltip..FONT_COLOR_CODE_CLOSE.." )";
		end

		local unitLevel = UnitLevel("player");
		unitLevel = max(unitLevel, 20);
		local magicResistanceNumber = resistance/unitLevel;
		if ( magicResistanceNumber > 5 ) then
			resistanceLevel = RESISTANCE_EXCELLENT;
		elseif ( magicResistanceNumber > 3.75 ) then
			resistanceLevel = RESISTANCE_VERYGOOD;
		elseif ( magicResistanceNumber > 2.5 ) then
			resistanceLevel = RESISTANCE_GOOD;
		elseif ( magicResistanceNumber > 1.25 ) then
			resistanceLevel = RESISTANCE_FAIR;
		elseif ( magicResistanceNumber > 0 ) then
			resistanceLevel = RESISTANCE_POOR;
		else
			resistanceLevel = RESISTANCE_NONE;
		end
		frame.tooltip2 = format(RESISTANCE_TOOLTIP_SUBTEXT, _G["RESISTANCE_TYPE"..frame:GetID()], unitLevel, resistanceLevel);

		-- local scale = 1.5
		frame:SetParent(DejaClassicStatsPane)
		-- frame:SetScale(1.5)
		-- frame:SetWidth( (DCS_FrameWidth / scale) - DCS_RframeInset)
		if (i==1) then
			frame:SetPoint("TOPLEFT", DCSDefensiveStatsHeader, "BOTTOMLEFT", 150,0)--Not TOP and BOTTOM becasue of a parenting issue that centers it over the Characterframe...
		else
			frame:SetPoint("TOPLEFT", _G["MagicResFrame"..(i-1)], "BOTTOMLEFT", 0,0)
		end
		--print(tooltipText)
	end

end